      program main
      include 'constants'
      include 'commonspec'
      
      REAL, DIMENSION(nlon,nlat,12) :: LAI_MODIS  !  LAI from MODIS (12 months)
      REAL, DIMENSION(nlon,nlat,12) :: z0M_land   !  surface roughness for momentum over land (12 months)
      REAL, DIMENSION(nlon,nlat,12) :: z0H_land   !  surface roughness for heat over land (12 months)
      REAL, DIMENSION(nlon,nlat,12) :: Rac0       !  in-canopy transfer resistance factor (12 months)
      REAL, DIMENSION(nlon,nlat,12) :: Rcutd0_O   !  Rcutd0_O3/LAI^0.25  = cuticle resist. factor for O3 (dry)
      REAL, DIMENSION(nlon,nlat,12) :: Rcutw0_O   !  Rcutw0_O3/LAI^0.5   = cuticle resist. factor for O3 (wet)
      REAL, DIMENSION(nlon,nlat,12) :: Rcutd0_S   !  Rcutd0_SO2/LAI^0.25 = cuticle resist. factor for SO2 (dry)
      
      REAL, DIMENSION(nlon,nlat) :: zref          !  reference height
      REAL, DIMENSION(nlon,nlat) :: ustar         !  friction velocity (m/s)
      REAL, DIMENSION(nlon,nlat) :: T             !  air temperature (K)
      REAL, DIMENSION(nlon,nlat) :: ps_atm        !  surface pressure (atm)
      REAL, DIMENSION(nlon,nlat) :: Tsoil         !  soil temperature (K)
      REAL, DIMENSION(nlon,nlat) :: RH            !  relative humidity (%)
      REAL, DIMENSION(nlon,nlat) :: TC            !  T in Celsius
      REAL, DIMENSION(nlon,nlat) :: invT298       !  1/T - 1/298  (K-1)
      REAL, DIMENSION(nlon,nlat) :: U10           !  10 meter WIND SPEED (m/s)
      REAL, DIMENSION(nlon,nlat) :: U10sq         !  10 meter WIND SPEED SQUARED (m^2/s^2)
      REAL, DIMENSION(nlon,nlat) :: dew           !  dew frequency (0-1)
      REAL, DIMENSION(nlon,nlat) :: air           !  air number density (cm-3)
      REAL, DIMENSION(nlon,nlat) :: rho_air       !  air density in kg m-3 (=~1.18 kg m-3 at 298 K, 1 atm)
      REAL, DIMENSION(nlon,nlat) :: Tfactor       !  T-dependence from Zhang et al 2003
      REAL, DIMENSION(nlon,nlat) :: ocean         !  ocean fraction (with sea ice)
      REAL, DIMENSION(nlon,nlat) :: oc            !  ocean fraction (without sea ice)
      REAL, DIMENSION(nlon,nlat) :: vland         !  vegetated land fraction
      REAL, DIMENSION(nlon,nlat) :: LAI           !  current leaf area index (LAI, m^2/m^2)
      REAL, DIMENSION(nlon,nlat) :: LAI_land      !  LAI of land fraction of grid cell
      REAL, DIMENSION(nlon,nlat) :: seaice        !  sea ice
      REAL, DIMENSION(nlon,nlat) :: bare          !  bare soil fraction
      REAL, DIMENSION(nlon,nlat) :: vege          !  vegetated land fraction
      REAL, DIMENSION(nlon,nlat) :: dry           !  dry fraction (no dew, no rain)
      REAL, DIMENSION(nlon,nlat) :: wet           !  (1 - dry)
      REAL, DIMENSION(nlon,nlat) :: fsnow_soil    !  snow fraction for soil
      REAL, DIMENSION(nlon,nlat) :: fsnow_veg     !  snow fraction for vegetation
      REAL, DIMENSION(nlon,nlat) :: Pcov          !  precipitation fraction
      REAL, DIMENSION(nlon,nlat) :: soilpH        !  soil pH
      REAL, DIMENSION(nlon,nlat) :: Charnock      !  Charnck number 
      REAL, DIMENSION(nlon,nlat) :: KinVisc       !  kinematic viscosity of air (m2 s-1)
      REAL, DIMENSION(nlon,nlat) :: D_H2O2        !  gas-phase diffusivity for H2O2 (m2 s-1)
      REAL, DIMENSION(nlon,nlat) :: z0M_sea       !  roughness length for momentum over sea (m)
      REAL, DIMENSION(nlon,nlat) :: z0H_sea       !  roughness length for heat over sea (m)
      REAL, DIMENSION(nlon,nlat) :: z0M           !  combined roughness length for momentum (land & ocean) (m)
      REAL, DIMENSION(nlon,nlat) :: z0H           !  combined roughness length for heat (land & ocean) (m)
      REAL, DIMENSION(nlon,nlat) :: wstar         !  free convection velocity scale (m/s)
      REAL, DIMENSION(nlon,nlat) :: wind          !  wind speed (m/s)
      REAL, DIMENSION(nlon,nlat) :: Q_0v          !  virtual temperature flux
      REAL, DIMENSION(nlon,nlat) :: Obukhov       !  Monin-Obukhov length (m)
      REAL, DIMENSION(nlon,nlat) :: CMH           !  square root of transfer coefficient for momentum
      REAL, DIMENSION(nlon,nlat) :: psi1,psi2     !  local
      
      REAL, DIMENSION(nlon,nlat) :: Ra            !  aerodynamic resistance (s/m)
      REAL, DIMENSION(nlon,nlat) :: Rb_H2O2       !  quasi-laminar BL resistance (s/m)
      REAL, DIMENSION(nlon,nlat) :: Rs            !  stomatal resistance for H2O (s/m)
      REAL, DIMENSION(nlon,nlat) :: RgO           !  ground resistance for O3 (s/m)
      REAL, DIMENSION(nlon,nlat) :: InvRgSol      !  ground conductance for soluble species (s/m)
      REAL, DIMENSION(nlon,nlat) :: Rac1          !  in-canopy aero. resistance factor
      REAL, DIMENSION(nlon,nlat) :: Rcutd1_O3     !  dry cuticular resistance factor for O3
      REAL, DIMENSION(nlon,nlat) :: Rcutw1_O3     !  wet cuticular resistance factor for O3
      REAL, DIMENSION(nlon,nlat) :: Rcutd1_SO2    !  dry cuticular resistance factor for SO2
      REAL, DIMENSION(nlon,nlat) :: Rac           !  in-canopy resistance (s/m)
      REAL, DIMENSION(nlon,nlat) :: Rcutd_O3      !  cuticular resistance (dry case) for O3 (s/m)
      REAL, DIMENSION(nlon,nlat) :: Rcutd_SO2     !  cuticular resistance (dry case) for SO2 (s/m)
      REAL, DIMENSION(nlon,nlat) :: Rcutw_O3      !  cuticular resistance (wet case) for O3 (s/m)
      REAL, DIMENSION(nlon,nlat) :: RgS0          !  ground resistance (dry) for SO2 (s/m)
      REAL, DIMENSION(nlon,nlat) :: RcutO         !  cuticular resistance for O3 (s/m)
      REAL, DIMENSION(nlon,nlat) :: RcutS         !  cuticular resistance for SO2 (s/m)
      REAL, DIMENSION(nlon,nlat) :: RgO_snow      !  snow-covered ground resistance for O3 (s/m)
      REAL, DIMENSION(nlon,nlat) :: RcutO_snow    !  snow-covered cuticular resistance for O3 (s/m)
      REAL, DIMENSION(nlon,nlat) :: Rc_snow_SO2   !  snow-covered canopy resistance for SO2 (s/m)
      REAL, DIMENSION(nlon,nlat) :: Rc_snow_O3    !  snow-covered canopy resistance for O3 (s/m)
      REAL, DIMENSION(nlon,nlat) :: Rc_snow       !  snow-covered canopy resistance (s/m)
      
      REAL, DIMENSION(nlon,nlat,nbcon) :: Rab     !  Ra + Rb  (s/m)
      REAL, DIMENSION(nlon,nlat,nbcon) :: Rg      !  ground resistance (s/m)
      REAL, DIMENSION(nlon,nlat,nbcon) :: Rgwater !  resistance of water surfaces (s/m)
      REAL, DIMENSION(nlon,nlat,nbcon) :: Rm      !  mesophyll resistance (s/m)
      REAL, DIMENSION(nlon,nlat,nbcon) :: Rc      !  canopy resistance (s/m)
      REAL, DIMENSION(nlon,nlat,nbcon) :: Rcut    !  cuticular resistance (s/m)
      REAL, DIMENSION(nlon,nlat,nbcon) :: Henry   !  Henry's law constant (M atm-1)
      REAL, DIMENSION(nlon,nlat,nbcon) :: vd      !  deposition velocity (cm/s)
      
      REAL, DIMENSION(nlon,nlat,nveg) :: fveg     !  fraction of cell covered by veg types

c  MOHYCAN  PFTs :
c    1-Needleleaf evergreen	2-Needleleaf deciduous	3-Broadleaf evergreen	
c    4-Broadleaf deciduous	5-Shrub			6-Crop			
c    7-Grass	               (8-Palm)   
      
c ****************************************************************************************************

      month = 7     !   month (here July)
      
      UT = 18.      !   Universal time

      
c READ THE MODEL SPECIES, CALCULATE DIFFUSION COEFFICIENTS & READ MODEL-DEPENDENT DEPOSITION PARAMETERS
      call read_species

c LEAF ARE INDEX (m^2/m^2) :  read monthly climatological LAI from MODIS
      open(1,file='INPUT/LAI_MODIS_climato_0.5x0.5',status='old',
     $                                             form='unformatted')
      do m=1,12
        read(1) LAI_MODIS(:,:,m)
      end do
      close (1)
      LAI(:,:) = LAI_MODIS(:,:,month)

***** VEGETATION TYPES (FRACTIONS) ****
      fveg(:,:,:) = 0.
      vland(:,:) = 0.
      open(1,file='INPUT/PFTs_MEGAN_0.5',status='old',
     $                                   form='unformatted')
      DO iveg = 1, 7
        read(1) fveg(:,:,iveg)
	vland(:,:) = vland(:,:) + fveg(:,:,iveg)  !  vegetated area cover
      END DO
      close (1)

c OCEAN FRACTIONS
      open(1,file='INPUT/ocean_daac.0.5',status='old',
     $                                             form='unformatted')
      read(1) ocean(:,:)
      close (1)
      
c SOIL pH
      soilpH(:,:) = 5.
      open(1,file='INPUT/SoilpH.0.5',status='old',
     $                                             form='unformatted')
      read(1) soilpH(:,:)
      close (1)
c------------------------------------

c  THE FOLLOWING FIELDS SHOULD BE PROVIDED BY THE USER
c    (HERE, TYPICAL VALUES ARE ASSUMED)

c SEA-ICE FRACTION
      seaice(:,:) = 0.
        
c AIR TEMPERATURE (K)   
      T(:,:)=298.

c AIR NUMBER DENSITY (cm-3)
      air(:,:)=2.45e19
      
c AIR DENSITY (kg m-3)
      rho_air(:,:)=1.18
      
c SURFACE PRESSURE (atm)
      ps_atm(:,:)=1.

c RELATVE HUMIDITY (%)
      RH(:,:)=85.
      
c SOIL TEMPERATURE (K)
      Tsoil(:,:) = 293.
      
c WIND SPEED at 10 m (m/s)
      U10(:,:)=5.    !   wind speed at 10 m (m/s)
      
c WIND SPEED SQUARED at 10 m (m/s)
      U10sq(:,:)=35. !   square of wind speed at 10 m (m^2/s^2)
      
c  PRECIPITATION FRACTION (unitless)
      Pcov(:,:) = 0.08

c  SNOW FRACTIONS
      fsnow_soil(:,:)=0.             !  snow fraction for soil
      fsnow_veg(:,:)=0.              !  snow fraction for vegetation

c CHARNOCK NUMBER 
      Charnock(:,:) = 15.E-3
      
c VIRTUAL TEMPERATURE FLUX (>0 when from surface to atmosphere, i.e. daytime over land)
      Q_0v(:,:) = 0.2
      
c REFERENCE HEIGHT  (should be the altitude of 1st model level)
      zref(:,:) = 10.   !  in meters
      
c STOMATAL RESISTANCES (s/m)
c    NOTE : the stomatal resistances can be calculated using program Stom_resistance.f
      Rs(:,:)=1./(0.01*vland(:,:) + 1.E-10*(1.-vland(:,:)))   !   100 s/m over vegetation, 1E10 over sea

c DEW FREQUENCY (unitless)
c    NOTE : the dew frequency can be calculated using program Stom_resistance.f
      dew(:,:) = 0.1
      
c-----------------------------------
c  calculate surface roughness for momentum (land) and PFT-dependent factors
c  for the calculation of cuticular and in-canopy transfer resistances

      CALL surf_rough(LAI_MODIS,vland,fveg,
     $                z0M_land,Rac0,Rcutd0_O,Rcutw0_O,Rcutd0_S)

c IN-CANOPY TRANSFER RESISTANCE FACTOR
      Rac1(:,:) = Rac0(:,:,month)
      
c dry cuticular resistance factor for O3 (i.e. Rcutd0_O / LAI**0.25 )
      Rcutd1_O3(:,:) = Rcutd0_O(:,:,month)
      
c wet cuticular resistance factor for O3 (i.e. Rcutw0_O / LAI**0.5 )
      Rcutw1_O3(:,:) = Rcutw0_O(:,:,month)
      
c dry cuticular resistance factor for SO2 (i.e. Rcutd0_S / LAI**0.25 )
      Rcutd1_SO2(:,:) = Rcutd0_S(:,:,month)
      
c **************** AERODYNAMIC RESISTANCE  **************************

c  surface roughness for heat (over land) (m)
      z0H_land(:,:,:) = z0M_land(:,:,:) / 10. 

c  free convection velocity scale (m/s)
      wstar(:,:)=(1000.*9.81*abs(Q_0v(:,:))/T(:,:))**0.33333

c  wind (m/s)
      wind(:,:)=SQRT(U10(:,:)**2 + wstar(:,:)**2)  

c kinematic viscosity of air (m2 s-1) (= 1.5E-5 m2/s  at 15 C)
      KinVisc(:,:)=(1.458e-06 * (T(:,:)**1.5)) / (T(:,:) + 110.4) 
     $                    / rho_air(:,:)
      
c initial guess of friction velocity (m s-1)
      ustar(:,:)  = 0.5
      
      DO ITER=1,5  !   iterate for u* & Obukhov computation

c CHARNOCK RELATIONSHIP : z0 = alpha * u^2/g,
        z0M_sea(:,:) = 0.11*KinVisc(:,:)/ustar(:,:) +
     $               Charnock(:,:) * (ustar(:,:)**2) / 9.81
        z0H_sea(:,:) = 0.40*KinVisc(:,:)/ustar(:,:)
	
c  overall surface roughness length (m)
        z0M(:,:)=z0M_land(:,:,month)*(1.-ocean(:,:))
     $         + z0M_sea(:,:)*ocean(:,:)
        z0H(:,:)=z0H_land(:,:,month)*(1.-ocean(:,:))
     $         + z0H_sea(:,:)*ocean(:,:)
     
c Monin-Obukhov length (meters)  N.B.  stable conditions : Obukhov > 0 

        Obukhov(:,:)=-T(:,:)*(ustar(:,:)**3) / (0.41*9.81*Q_0v(:,:))
	
c  re-calculation of friction velocity (ustar)
c  first calculate transfer coefficient for momentum (or rather, its square root)
        call calcpsiM( (zref+z0M)/Obukhov   , psi1)
        call calcpsiM( z0M(:,:)/Obukhov   , psi2)
        CMH(:,:)=0.41/(alog((zref+z0M)/z0M) - psi1 + psi2 )
      
        ustar(:,:)=CMH(:,:) * wind(:,:)
      
        ustar(:,:)=max(0.01,ustar(:,:))

      END DO
      
      call calcpsiH( (zref+z0M)/Obukhov    , psi1)
      call calcpsiH(  z0H(:,:)/Obukhov   , psi2)

c aerodynamic resistance in s m-1
      Ra(:,:)= ( alog((zref+z0M)/z0H) - psi1 + psi2 )/(0.41*ustar)

      
c **********  QUASI_LAMINAR BOUNDARY LAYER RESISTANCE  ************** 
     
      Dg298_H2O2 = 1.54E-5            !   gas-phase diffusion coefficient for H2O2 (298 K)
     
c gas-phase diffusivity for H2O2 
      D_H2O2(:,:)= Dg298_H2O2/ ps_atm(:,:) * (T(:,:)/298.)**1.75 

c   Rb = (2/(k*ustar))*(nu/(Pr*Di))^(2/3)
c             with nu the kinematic viscosity, k=0.41, Pr=0.72, Di = gaseous diffusivity

c          noting that 2/(0.41 * 0.72^0.6667) = 6.07 :
      Rb_H2O2(:,:)=(6.07/ustar(:,:))*(KinVisc(:,:)/D_H2O2(:,:))**(2./3)

     
c **********  SURFACE RESISTANCES OVER OCEAN  ************** 
    
      call Rgsea_all(Tsoil,U10,U10sq, !  
     $                     Rgwater)   !  calculate surface resistance over ocean for all soluble species

c **********  SURFACE RESISTANCES OVER LAND  **************  
      
      invT298(:,:)=1./max(T,273.15) - 1./298.
      
      Rac(:,:)      = Rac1(:,:) / (ustar(:,:)**2) ! Zhang et al 2003, Eq. (7)
      
c  ocean fraction in grid cell
      oc(:,:)=max(0.,ocean(:,:)-seaice(:,:))
      
      LAI_land(:,:)=LAI(:,:)/max(1.E-5,vland(:,:))
      LAI_land(:,:)=min(8.,LAI_land(:,:))
      
      vege(:,:)=vland(:,:)*min(1.,LAI_land(:,:)/3.) ! N.B. vland = sum of MEGAN's PFTs
      bare(:,:)=max(0.,1.-vege(:,:)-ocean(:,:))
      
      
      dry(:,:) = (1.-dew(:,:))*(1.-Pcov(:,:))  !  Pcov=rain fraction; dew = dew fraction
      wet(:,:)=1.-dry(:,:)
      
c cf. Zhang et al. 2003

      Tfactor(:,:)=max(1.,min(2.,exp(0.2*(-1.-TC(:,:)))))

      RgO(:,:)=1./ ( ocean(:,:)/2000.
     $              + vege(:,:)/200.
     $              + bare(:,:)/500. )
      RgO(:,:)=RgO(:,:) * Tfactor(:,:)
      RgO_snow(:,:)=2000.
      
      do k=1,nlon
      do i=1,nlat
        if(soilpH(k,i).LT.5.5) then    !   cf Kerkweg 2006
	  RgS0(k,i)=115.
	else if(soilpH(k,i).LT.7.3) then
	  RgS0(k,i)=65.
	else
	  RgS0(k,i)=25.
	endif
	 
	if(RH(k,i).LT.60.) then
	  RgS0(k,i)=RgS0(k,i)*3.41 - 85. + 1000.*exp(269.-T(k,i))
     $                            + 1.E5*max(0.,(40.-RH(k,i))/40.)
	endif
      end do
      end do
      
      RgS0(:,:)=max(25., RgS0(:,:))  !!! to avoid ridiculously low values over deserts
      
      Rcutd_O3(:,:) = max(1.E-9,Rcutd1_O3(:,:)   * Tfactor(:,:)
     $                         / (ustar(:,:)*exp(0.03*RH(:,:))) )
      Rcutw_O3(:,:) =max(1.E-9, Rcutw1_O3(:,:)  / (ustar(:,:)))
      RcutO(:,:) = 1./(dry(:,:)/Rcutd_O3(:,:)+wet(:,:)/Rcutw_O3(:,:))
      
      RcutO_snow(:,:)=2000.
      RcutO_snow(:,:)=max( RcutO_snow(:,:),RcutO(:,:) )  !!! seems reasonable to me
      
      Rcutd_SO2(:,:)= max(1.e-10,Rcutd1_SO2(:,:) * Tfactor(:,:)
     $                         / (ustar(:,:)*exp(0.03*RH(:,:))))
      RcutS(:,:)=1./( dry(:,:)/Rcutd_SO2(:,:)
     $              + (Pcov(:,:)/50. + dew(:,:)*(1.-Pcov(:,:))/100.)
     $              *(ustar(:,:)*SQRT(LAI_land(:,:)+1.E-20)) )
      
      do k=1,nlon
      do i=1,nlat
        if(LAI(k,i).LE.0.01.OR.
     $     RcutO(k,i).LE.1.E-9) then
	  RcutS(k,i)=1.E10
	  RcutO(k,i)=1.E10
	  RcutO_snow(k,i)=1.E10
	endif
      end do
      end do
      
      Rc_snow_O3(:,:)=1./( 1./(RgO_snow(:,:)+Rac(:,:))   !  as in Zhang 2003
     $                   + 1./RcutO_snow(:,:) )
	  
      Rc_snow_SO2(:,:)=min(1.E5,10.**(-0.09*TC(:,:)+2.4))  !  Kerkweg 2006
      
      Dg298_H2O  = 2.51E-5            !   gas-phase diffusion coefficient for H2O (298 K)
      
c **********  COMBINE RESISTANCES FOR ALL SPECIES **************   
      
      do n=1,nbcon
      
        if(H298(n)+f0reac(n).NE.0.) then
	  
	  
	  Rab(:,:,n)=Ra(:,:)
     $              +Rb_H2O2(:,:) * (Dg298_H2O2/Dg298(n))**0.66667
	  
	  Henry(:,:,n)=H298(n)*exp(BHenry(n)*invT298(:,:))

c mesophylic resistance (Wesely 1989 Eq. (6))
	  Rm(:,:,n)=1./(Henry(:,:,n)/3000.*f1fact(n) 
     $                 + 100*f0reac(n))  !  =0.01 s m-1 for O3

c ground resistance (Eq. (9))
c          InvRgSol(:,:)= oc(:,:)/20.            !  Wesely: SO2 ground resistance=20 s m-1 for ocean
          InvRgSol(:,:)= oc(:,:)/Rgwater(:,:,n)  !  new param. for soluble cpds (Rgsea_all)
     $       + ( seaice(:,:)/min(500.,max(100.,70.*(2.-TC(:,:))))
     $           + (1.-ocean(:,:))* Tfactor(:,:) *
     $          (  dry(:,:)/RgS0(:,:)
     $           + Pcov(:,:)/50. + dew(:,:)*(1.-Pcov(:,:))/100. ) )
     $           * 1.E-5*Henry(:,:,n)*f1fact(n)
     
          Rg(:,:,n)=1./( InvRgSol(:,:)
     $                 + f0reac(n)/RgO(:,:) )
	  
c cuticular resistance (analoguous to ground)
          Rcut(:,:,n)=1./( 1.E-5*Henry(:,:,n)/RcutS(:,:)*f1fact(n)
     $                   + f0reac(n)/RcutO(:,:) )
	  
********************
*** TEST  T. Nguyen expressions for Rm and Rcut (with the mistake in Rcut!):
c          Rm(:,:,n)=1./(Henry(:,:,n)/(50*0.082*T(:,:)) + 100*f0reac(n))
c	  Rcut(:,:,n)=1./( 1.e-4*Henry(:,:,n)/(0.082*T(:,:))
c     $                   + f0reac(n) )
********************
          
	  Rm(:,:,n)=Rm(:,:,n)+Rs(:,:)*Dg298_H2O/Dg298(n)
	  
	  
          Rc(:,:,n)=1./( 1./Rm(:,:,n)           
     $                 + 1./(Rg(:,:,n)+Rac(:,:))
     $                 + 1./Rcut(:,:,n) )       
	  
	  Rc_snow(:,:)=1./
     $           ( min(1.,1.E-5*Henry(:,:,n))/Rc_snow_SO2(:,:) ! Wesely, modified for high solub specie
     $                   + f0reac(n)/Rc_snow_O3(:,:) )
	  
	  vd(:,:,n)=
     $         100./(Rab(:,:,n)+Rc(:,:,n)) * (1.-fsnow_veg(:,:))
     $        +100./(Rab(:,:,n)+Rc_snow(:,:)) * (fsnow_veg(:,:))
          print*,spec(n),' vd=',vd(1,1,n)
	  
	endif
	
      end do
         
      
      end
      
c ======================================================================

      subroutine read_species
      include 'constants'
      include 'commonspec'
      character*100 line,string
      character*15 comp
      character*1 car
      
      INTEGER nbatom
      PARAMETER (nbatom=11)     !  nb of different atoms considered
      
      INTEGER, DIMENSION(nbatom)   ::  natom     !  number of atoms of each type in a molecule
      REAL, DIMENSION(nbatom)      ::  atmat     !  atomic masses
      REAL, DIMENSION(nbatom)      ::  volat     !   atomic diffusion volumes (cm3 mol-1)
      REAL, DIMENSION(nbatom)      ::  liqat     !   liquid molar volumes (cm3 mol-1)
      
      CHARACTER, DIMENSION(nbatom) ::  atom      
      data atom /  'H ', 'C ', 'N ', 'O ', 'F ', 'Cl', 'S ','Rn','Pb',
     $             'R ', 'D ' /   !  R=ring ; D=double bond
      
      data atmat/  1.  ,  12.,  14.,  16.,  18.,   35.,  32.,222.,210.,
     $             0.  ,   0. /
      
c Reid et al. (1966) atomic diffusion volume (dimensionless)
      data volat/  2.31, 15.9, 4.54, 6.11, 10.0,  21.0, 22.9, 20., 20.,
     $           -18.3 ,   0. /
      
c Schroeder liquid molar volume
      data liqat/  7.0 ,  7.0, 7.0 , 7.0 , 10.5,  24.5, 21.0, 20., 20.,
     $            -7.0 ,  7.0/

c READ MODEL SPECIES FILE + CALCULATE GAS- AND LIQUID- PHASE DIFFUSIVITIES (at 298 K)

      open(4,file='INPUT/MAGRITTE.spc',status='old')
      do ll=1,4
        read(4,*)
      end do
      
      ico=0
      
      
      do ll=1,1000
      
      read(4,'(a15,1x,a40)') comp,line
      if(line(1:1).EQ.'=') line=line(2:)
      if(line.eq.' ') goto 3
      
      ico=ico+1
      spec(ico)=comp
      totmass=0.
      totdiffv=0.   !  molecular diffusion volume (cm3 mol-1)
      totliqv=0.    !  molecular liquid volumes (cm3 mol-1)
      lpv=index(line,';')-1   !  semicolon delimiter
      if(lpv.LT.0) stop 'lpv.LT.0'
      if(line(lpv+1:lpv+3).EQ.';{+') then  !  double bonds or rings are present
        lend=index(line,'}')-1
        line=line(:lpv)//line(lpv+3:lend)//';'
      endif

 10   continue
      lpv=index(line,';')-1
      if(line(:lpv).LE.' ') goto 5
      lpl=index(line,'+')-1
      if(lpl.GT.0) then
        lpl=min(lpv,lpl)
      else
        lpl=lpv
      endif
      string=line(:lpl)
      do iatom=1,nbatom
        ipos=index(string,atom(iatom))
	if(ipos.ne.0) then
	  if(string(:ipos-1).LE.' ') then
	    a=1
	  else
	    read(string(:ipos-1),*) a
	  endif
	  totmass=totmass+a*atmat(iatom)
	  totdiffv=totdiffv+a*volat(iatom)
	  totliqv=totliqv+a*liqat(iatom)
	endif
      end do
      line=line(lpl+2:)
      goto 10

  5   continue


c adjust diffusion volumes for few cases (Reid, 87 , cf Tang et al 2014)
      if(comp(:4).EQ.'H2O ') totdiffv=13.1
      if(comp(:3).EQ.'SO2' ) totdiffv=41.8
      if(comp(:4).EQ.'tNH3') totdiffv=20.7

c calculate gas-phase diffusivity at 298 K, 1 atm (m2 s-1) according to Fuller et al. (1966):
      Dg298(ico)=1.e-7 * (298.**1.75) *SQRT(1./totmass+1./29.)
     $          /(totdiffv**0.33333 + 19.7**0.33333)**2

c  H2O :  1.e-7 * (298.**1.75) * SQRT(1/18. + 1/29.)
c               /(13.1**0.33333 + 19.7**0.33333)**2   !   = 2.51E-5 @ 298 K
c  H2O :  1.e-7 * (273.15**1.75) * SQRT(1/18. + 1/29.)
c               /(13.1**0.33333 + 19.7**0.33333)**2   !   = 2.15E-5 @ 273.15 K

c  O3  :  1.e-7 * (298.**1.75) * SQRT(1/48. + 1/29.)
c               /(16.44**0.33333 + 19.7**0.33333)**2   !   =

c  SO2 :  1.e-7 * (298.**1.75) * SQRT(1/64. + 1/29.)
c               /(41.1**0.33333 + 19.7**0.33333)**2    !   =

c  H2O2 :  1.e-7 * (298.**1.75) * SQRT(1/34. + 1/2.9)
c               /(14.9**0.33333 + 19.7**0.33333)**2    !   =

c  MVK  : 1.e-7 * (298.**1.75) * SQRT(1/70. + 1/29.)
c               /(83.36**0.33333 + 19.7**0.33333)**2   !   =

c      Dg298(ico) = 2.178E-5 * SQRT(18./totmass)  !  Graham's law + H2O expmtl value
       
c special cases (Tang et al. 2014 )
       if(comp(:3).EQ.'SO2'     ) Dg298(ico) = 1.252E-5
       if(comp(:4).EQ.'tNH3'    ) Dg298(ico) = 2.344E-5
c       if(comp(:3).EQ.'O3 '    ) Dg298(ico) = 1.690E-5  !  Massman 1998
       if(comp(:4).EQ.'H2O2'    ) Dg298(ico) = 1.55E-5
       if(comp(:4).EQ.'HNO3'    ) Dg298(ico) = 1.14E-5
       if(comp(:3).EQ.'NOx'     ) Dg298(ico) = 1.450E-5  !  assume 80% NO2 and 20% NO
       if(comp(:3).EQ.'HONO'    ) Dg298(ico) = 1.278E-5
c special cases (Tang et al. 2015)
       if(comp(:3).EQ.'CH4'     ) Dg298(ico) = 2.210E-5
       if(comp(:5).EQ.'CH3OH'   ) Dg298(ico) = 1.658E-5
       if(comp(:8).EQ.'CH3COCH3') Dg298(ico) = 1.059E-5
       if(comp(:3).EQ.'MEK'     ) Dg298(ico) = 0.908E-5
       if(comp(:5).EQ.'HCOOH'   ) Dg298(ico) = 1.526E-5
       if(comp(:7).EQ.'CH3COOH' ) Dg298(ico) = 1.237E-5
       if(comp(:5).EQ.'C2H5COOH') Dg298(ico) = 0.947E-5

       if(comp(:3).EQ.'NO ')      Dg298(ico) = 1.802E-5  ! Tang et al. 2014
       if(comp(:3).EQ.'NO2')      Dg298(ico) = 1.361E-5  ! Tang et al. 2014
       if(comp(:3).EQ.'NO3')      Dg298(ico) = 1.224E-5  ! Tang et al. 2014
       if(comp(:4).EQ.'N2O5')     Dg298(ico) = 0.865E-5  ! Tang et al. 2014
       if(comp(:3).EQ.'OH ')      Dg298(ico) = 2.342E-5  ! Tang et al. 2014
      
      
c calculate liquid-phase diffusivity (here m2 s-1) as reported in Johnson :
      eta_s298 = 0.965e-3    !  dynamic viscosity of sea water at 298 K (Hardy)
      Dw298(ico) = 7.4e-15*298.*SQRT(2.6*18.01)
     $                 / (eta_s298*totliqv**0.6)

      write(6,*) comp,' Dg=',Dg298(ico)
      
  2   continue
  
      end do

  3   continue
      
      close (4)
      
      
      
c ----------------------------------------------------------------------------
c  READ HENRY'S LAW CONSTANTS AND DRY DEPOSITION PARAMETERS FOR ALL COMPOUNDS
      
      open(1,file='INPUT/deposition_parameters.txt',status='old')
      read(1,*)
      read(1,*)
      read(1,*)
      H298(:)=0.
      BHenry(:)=0.
      f0reac(:)=0.
      f1fact(:)=1.
      do ll=1,999
        read(1,*,end=1) comp,xH298,B,f0,f1
	nok=0
	do n=1,nbcon
	  if(comp.EQ.spec(n)) nok=n
	end do
	if(nok.NE.0) then
	  H298(nok)=xH298
	  BHenry(nok)=B
	  f0reac(nok)=f0
	  f1fact(nok)=f1
	endif
      end do
  1   close (1)
      
      end

c----------------------------------------------------------------------
      
      subroutine Rgsea_all(Tsoil,U10,U10sq,
     $                     Rgwater)
      include 'constants'
      include 'commonspec'
      REAL, DIMENSION(nlon,nlat) :: Tsoil         !  soil (or water) temperature (K)
      REAL, DIMENSION(nlon,nlat) :: U10           !  10 meter WIND SPEED (m/s)
      REAL, DIMENSION(nlon,nlat) :: U10sq         !  10 meter WIND SPEED SQUARED (m^2/s^2)
      
      REAL, DIMENSION(nlon,nlat) :: TC            !  T in Celsius
      REAL, DIMENSION(nlon,nlat) :: invT298       !  1/T - 1/298  (K-1)
      REAL, DIMENSION(nlon,nlat) :: eta_w         !  dynamic viscosity of pure water [kg m-1 s-1] 
      REAL, DIMENSION(nlon,nlat) :: eta_s         !  dynamic viscosity of seawater (centipoise)
      REAL, DIMENSION(nlon,nlat) :: Dw            !  diffusion coefficient in water (m2 s-1)
      REAL, DIMENSION(nlon,nlat) :: Sc            !  Schmidt number
      REAL, DIMENSION(nlon,nlat) :: kl            !  liquid-phase transfer velocity (cm s-1)
      REAL, DIMENSION(nlon,nlat) :: KHRT          !  KH*R*T, with KH in M atm-1, R=8.205e-2 l atm K-1 mol-1
      
      REAL, DIMENSION(nlon,nlat,nbcon) :: Rgwater !  resistance of water surfaces (s/m)
      
      REAL, DIMENSION(9)          :: eta_Hardy     ! dynamic viscos. (cP) of seawater at S=35 (Johnson 2010)
                                                   ! for T(Celsius)=-5, 0, ..., 35
      REAL, DIMENSION(9)          :: t_hardy       ! temperatures (C) corresponding to eta_Hardy vector
      data eta_Hardy/ 2.244,1.88,1.605,1.390,1.219,
     $                1.08,0.965,0.870,0.786 /
      data t_hardy/ -5.,0.,5.,10.,15.,20.,25.,30.,35. /
      
c surface water temperature in Celsius
      TC(:,:)=max(-10.,Tsoil(:,:)-273.15)
      
      invT298(:,:)=1./max(Tsoil(:,:),273.15) - 1./298.

c calculate the liquid-phase transfer velocity (Johnson 2010)

c dynamic viscosity of pure water [kg m-1 s-1] (from Johnson 2010)
c    N.B. eta_w at 20 C = ~1 centipoise i.e. 1.e-3 kg m-1 s-1
      eta_w(:,:)=1.e-3 *(TC(:,:)+246.)
     $          /(0.05594*TC(:,:)**2+5.2842*TC(:,:)+137.37)

      do k=1,nlon
      do i=1,nlat
        t=TC(k,i)
	do it=2,8
	  if(t.LT.t_hardy(it)) goto 1
	end do
c dynamic viscosity of seawater (centipoise) at S=35 from Hardy (cf. Johnson 2010)
  1     eta_s(k,i)=eta_Hardy(it-1)+(eta_Hardy(it)-eta_Hardy(it-1))*
     $                             (t-t_Hardy(it-1))/5.
      end do
      end do
      eta_s(:,:)=eta_s(:,:)/(0.965)   !   normalize by value at 298 K

      S=35.          !  salinity psu
      rho_w=1025.    !   typical oceanic water density in kg m-3 (for S=35)
      
      do n=1,nbcon
        
	if(H298(n).NE.0.) then
	
c diffusion coefficient in water (m2 s-1)
          Dw(:,:)=Dw298(n)*(Tsoil(:,:)/298.)/(eta_s(:,:))
	  
c Schmidt number
          Sc(:,:)=eta_w(:,:)/(rho_w*Dw(:,:))    !  ~1.e-3/(1025.*1.6e-9) = ~600
	  
c liquid-phase transfer velocity (cm s-1)
          kl(:,:)=(0.222*U10sq(:,:)+0.333*U10(:,:))  ! U10 in m s-1, U10 in m2 s-2
     $           *SQRT(600./Sc(:,:)) / 3600.
     
	  
c KH*R*T, with KH in mol l-1 atm-1, R = 8.205e-2 l atm K-1 mol-1
          KHRT(:,:)=H298(n)*exp(BHenry(n)*invT298(:,:))
     $             * 8.205e-2 * Tsoil(:,:)

c correct for acid dissociation (assuming pH = 7)
          if(spec(n).EQ.'HCOOH')   KHRT(:,:)=KHRT(:,:)*(1.+1.8e-4/1.e-7)
	  if(spec(n).EQ.'CH3COOH') KHRT(:,:)=KHRT(:,:)*(1.+1.7e-5/1.e-7)
	  if(spec(n).EQ.'C2H5COOH')KHRT(:,:)=KHRT(:,:)*(1.+1.3e-5/1.e-7)
          
	  Rgwater(:,:,n)=100./(kl(:,:)*KHRT(:,:))  !  in s m-1
	  
	endif
	
      end do
      
      end
      
c =============================================================================

      subroutine surf_rough(LAI,vland,fveg,
     $                      z0_land,Rac0,Rcutd0_O,Rcutw0_O,Rcutd0_S)

c  calculate PFT-dependent fields (surface roughness and resistances)
c  at 0.5x0.5 degree resolution for use in dry deposition model
      
      include 'constants'
      
      
c surface roughness (meters): use max value as in ECMWF IFS
c  use seasonality of NPP to estimate seasonal variation
c 
c canopy aerodynamic resistance (Rac) from Zhang et al. 2002, 2003
c         Rac = Rac0 * LAI^0.25 / (ustar^2)
c   Rac0 = Rac0min + (LAI-LAImin)/(LAImax-LAImin) * (Rac0max - Rac0min)
c  (illogical seasonal dependence introduced by Zhang et al. 2003)

c cuticular resistance factors for O3 and SO2 from Zhang et al. 2003
c         Rcutd = Rcutd0 / (exp(0.03RH) * LAI^0.25 * ustar)
c         Rcutw = Rcutw0 / (LAI^0.5 * ustar)

      REAL, DIMENSION(nveg) :: z0max,z0min,Rac0min,Rac0max
      REAL, DIMENSION(nveg) :: Rcutd0_O3,Rcutw0_O3,Rcutd0_SO2,Rgd_SO2
      REAL, DIMENSION(nlon,nlat,12)   :: LAI     ! LAI from MODIS
      REAL, DIMENSION(nlon,nlat,nveg) :: fveg    ! fraction of cell covered by veg types
      REAL, DIMENSION(nlon,nlat)      :: vland   ! fraction of cell covered by vegetation
      REAL, DIMENSION(nlon,nlat,12) :: z0_land   ! surface roughness (m) for momentum
      REAL, DIMENSION(nlon,nlat,12) :: Rac0      ! canopy resistance factor Rac0(t)
      REAL, DIMENSION(nlon,nlat,12) :: Rcutd0_O  ! Rcutd0_O3/LAI^0.25
      REAL, DIMENSION(nlon,nlat,12) :: Rcutw0_O  ! Rcutw0_O3/LAI^0.5
      REAL, DIMENSION(nlon,nlat,12) :: Rcutd0_S  ! Rcutd0_SO2/LAI^0.25
      REAL LAIx(12)
      
      REAL LAImax, LAImin, alpha, ustar, Rac
      INTEGER k,i,m,iveg,itime
      
c  MOHYCAN  PFTs :
c    1-Needleleaf evergreen	2-Needleleaf deciduous	3-Broadleaf evergreen	
c    4-Broadleaf deciduous	5-Shrub			6-Crop			
c    7-Grass	                8-Palm    

c roughness lengths for momentum
      data z0max/ 2.0,2.0,2.5,1.0,0.20,0.05,0.05/  ! similar to ECMWF IFS
      data z0min/ 2.0,0.4,2.5,0.4,0.05,0.02,0.05/  ! with some seasonal variation, cf. Zhang
      data Rac0max/ 100.,100.,250.,250.,60.,40.,40./
      data Rac0min/ 100., 60.,250.,100.,60.,10.,10./
c      data Rcutd0_O3/ 4000.,4000.,6000.,6000.,5000.,4000.,4000. / !  original values from Zhang 2003
c      data Rcutw0_O3/  200., 200., 400., 400., 300., 200., 200. / ! 
      data Rcutd0_O3/ 6000.,6000.,9000.,9000.,7500.,6000.,6000. /  !  my enhanced Rcut for O3
      data Rcutw0_O3/  300., 300., 600., 600., 450., 300., 300. /  ! 
      data Rcutd0_SO2/2000.,2000.,2500.,2500.,2000.,1500.,1000. /
      data Rgd_SO2  /  200., 200., 100., 200., 200., 200., 200. /
      
      z0_land(:,:,:)=0.
      Rac0(:,:,:)=0.
      Rcutd0_O(:,:,:)=0.
      Rcutw0_O(:,:,:)=0.
      Rcutd0_S(:,:,:)=0.
      
      DO k=1,nlon
      DO i=1,nlat
        LAImax=-9999.
        LAImin=1.E30
        DO m=1,12
	  IF(LAI(k,i,m).GT.LAImax) LAImax=LAI(k,i,m)
	  IF(LAI(k,i,m).LT.LAImin) LAImin=LAI(k,i,m)
	END DO
	
	IF(LAImax.GT.0.) THEN
	  
	  LAImax=max(LAImax,LAImin+0.001)
	  
	  IF( vland(k,i).GT.0.) THEN
	  
	    DO m=1,12
	      alpha=(LAI(k,i,m)-LAImin)/(LAImax-LAImin)
	      DO iveg=1,nveg
	       z0_land(k,i,m)=z0_land(k,i,m)+fveg(k,i,iveg)*
     $              (alpha*z0max(iveg)+(1.-alpha)*z0min(iveg))
               Rac0(k,i,m)=Rac0(k,i,m)+fveg(k,i,iveg)*
     $              (alpha*Rac0max(iveg)+(1.-alpha)*Rac0min(iveg))
               Rcutd0_O(k,i,m)=Rcutd0_O(k,i,m)+
     $              fveg(k,i,iveg)*Rcutd0_O3(iveg)
               Rcutw0_O(k,i,m)=Rcutw0_O(k,i,m)+
     $              fveg(k,i,iveg)*Rcutw0_O3(iveg)
               Rcutd0_S(k,i,m)=Rcutd0_S(k,i,m)+
     $              fveg(k,i,iveg)*Rcutd0_SO2(iveg)
	      END DO
	    END DO
	    LAIx(:)=max(0.01,LAI(k,i,1:12))
	    Rac0(k,i,:)=Rac0(k,i,:) * LAIx(:)**0.25
	    Rcutd0_O(k,i,:)=Rcutd0_O(k,i,:) / LAIx(:)**0.25
	    Rcutw0_O(k,i,:)=Rcutw0_O(k,i,:) / LAIx(:)**0.5
	    Rcutd0_S(k,i,:)=Rcutd0_S(k,i,:) / LAIx(:)**0.25
	    
	  ENDIF
	  
	ENDIF
	
      END DO
      END DO
      
      
      DO m=1,12
      
        z0_land(:,:,m)=z0_land(:,:,m) / (vland(:,:)+1.E-10)
	z0_land(:,:,m)=max(0.005, z0_land(:,:,m))   !   over deserts, z0 = 0.005 m
	
	Rac0(:,:,m)=Rac0(:,:,m) / (vland(:,:)+1.E-10)
	Rcutd0_O(:,:,m)=Rcutd0_O(:,:,m) / (vland(:,:)+1.E-10)
	Rcutw0_O(:,:,m)=Rcutw0_O(:,:,m) / (vland(:,:)+1.E-10)
	Rcutd0_S(:,:,m)=Rcutd0_S(:,:,m) / (vland(:,:)+1.E-10)
	
        DO k=1,nlon
        DO i=1,nlat
	  IF(Rcutd0_O(k,i,m).EQ.0.) Rcutd0_O(k,i,m)=1.E10
	  IF(Rcutw0_O(k,i,m).EQ.0.) Rcutw0_O(k,i,m)=1.E10
	  IF(Rcutd0_S(k,i,m).EQ.0.) Rcutd0_S(k,i,m)=1.E10
	END DO
        END DO
	
      END DO
      
      end


c --------------------------------------------------------------------
      subroutine calcpsiH(zeta,psi)
      include 'constants'
      real zeta(nlon,nlat),psi(nlon,nlat)
      real psi_unstable(nlon,nlat),psi_stable(nlon,nlat)
      real x(nlon,nlat)
      
      x(:,:)=(1.-16*zeta(:,:))**0.25  !  x1 = 1.43 ; x2 = 1.13 (land), 1.00008 (ocean)
      psi_unstable(:,:)=2*alog( (1.+x(:,:)**2)/2. ) ! psi1 = 0.841, psi2 = 0.259 (land), 1.6e-4 (ocean)
c      psi_stable(:,:)=-5.*zeta(:,:)   !  psi1 = -0.5, psi2 = -0.1 (land), -5E-5 (land)

c cf. IFS model 
      b=2./3
      c=5.
      d=0.35
      psi_stable(:,:)= - b*(zeta-(c/d))*exp(-d*zeta)
     $                 - ((1.+b*zeta(:,:))**1.5) - (b*c/d) + 1.

      do ilon=1,nlon
      do ilat=1,nlat
        if(zeta(ilon,ilat).LT.0.) then   !   UNSTABLE (zeta has same sign as Obukhov)
	  psi(ilon,ilat)=psi_unstable(ilon,ilat)
	else                             !   STABLE
	  psi(ilon,ilat)=psi_stable(ilon,ilat)
	endif
      end do
      end do
      
      end

c --------------------------------------------------------------------
      subroutine calcpsiM(zeta,psi)
      include 'constants'
      real zeta(nlon,nlat),psi(nlon,nlat)
      real psi_unstable(nlon,nlat),psi_stable(nlon,nlat)
      real x(nlon,nlat)
      parameter (pih=3.141592654/2.)
      
      x(:,:)=(1.-16*zeta(:,:))**0.25  !  x1 = 1.43 ; x2 = 1.13 (land), 1.00008 (ocean)
      psi_unstable(:,:) = pih - 2*atan(x)
     $                  + alog(((1.+x)**2)*(1.+x**2)/8.)

c cf. IFS model 
      b=2./3
      c=5.
      d=0.35
      psi_stable(:,:)= - b*(zeta-(c/d))*exp(-d*zeta)
     $                 - zeta - (b*c/d)

      do ilon=1,nlon
      do ilat=1,nlat
        if(zeta(ilon,ilat).LT.0.) then   !   UNSTABLE (zeta has same sign as Obukhov)
	  psi(ilon,ilat)=psi_unstable(ilon,ilat)
	else                             !   STABLE
	  psi(ilon,ilat)=psi_stable(ilon,ilat)
	endif
      end do
      end do
      
      end
